SystemOrganization addCategory: #'PetitPreprocessor-Core'!
SystemOrganization addCategory: #'PetitPreprocessor-Tests'!

TestCase subclass: #PPInfoParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Tests'!

!PPInfoParserTest methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 6/12/2012 11:12'!
testBasicInfo
	| nodes infos |
	nodes:= ('Lorem ipsum ' asParser, ('dolor' asParser token info),' sit amet' asParser) parse: 'Lorem ipsum dolor sit amet'.
	infos:= nodes second.
	self assert: infos start equals: 13.
	self assert: infos stop equals: 17.
	self assert: infos element start equals: infos start.
	self assert: infos element stop equals: infos stop.! !

!PPInfoParserTest methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 6/12/2012 11:19'!
testPreProcessedStreamInfo
	| nodes infos |
	nodes:= (('Lorem ipsum ' asParser, ('dolor' asParser token info),' sit amet' asParser) preProcess: '\[Sp\]' asRegex into: ' ') parse: 'Lorem[Sp]ipsum[Sp]dolor[Sp]sit[Sp]amet'.
	infos:= nodes second.
	self assert: infos start equals: 19.
	self assert: infos stop equals: 23.! !

PPAbstractParserTest subclass: #PPPreProcessingParserRegexTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Tests'!

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testBiggerReplacementThanMatching
	|preProcessingParser|	
	preProcessingParser := 'Troll' asParser preProcess: ('u' asRegex) into: 'll'. 
	self assert: (('Un' asParser, preProcessingParser, 'DeTroy' asParser) end matches: 'UnTrouDeTroy')! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testDecomposedEntry
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcess: 'T' asRegex into: ''.
	self assert: (preProcessingParser end matches: 'LibTelTluTle')! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testDecomposedEntryConsumed
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcess: 'T' asRegex into: ''.
	self assert:  ((preProcessingParser, 'yoyo' asParser) matches:  'LibTelTluTleyoyo')! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testDoNotConsumeIfContainedParserDoNotMatch
	|preProcessingParser|	
	preProcessingParser :=  'aNonMatchingString' asParser preProcess: 'T' asRegex into: ''.
	self assert: ((preProcessingParser optional, 'LibTelTluTle' asParser) matches: 'LibTelTluTle')! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testDoubleInclusion
	| preProcessingParser parser |
	preProcessingParser := ('Libellule' asParser preProcess: 'Y+' asRegex into: '') preProcess: 'T' asRegex  into: ''.
	parser :=  'yoyo' asParser, preProcessingParser, 'yoyo' asParser.
	self assert: (parser matches: 'yoyoLiYYbTeYYlTluTlYYeyoyo').! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testMixingDataToPreProcessAndNot
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcess: 'T' asRegex into: ''.
	self assert: (('DoNoTPreProcess' asParser, preProcessingParser, 'NotThisToo' asParser) matches: 'DoNoTPreProcessLibTelTluTleNotThisToo')! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testPreprocess
	| preProcessor |
	preProcessor := PPreRegexStreamPreProcessor new.
	preProcessor regex: 'T' asRegex .
	preProcessor replacement: ''.
	self assert: 'My String' equals: (preProcessor preProcess: 'MTy TStrTingT' asPetitStream ) contents ! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testPreprocessWithPreviousContent
	| aStream preProcessor |
	preProcessor := PPreRegexStreamPreProcessor new.
	preProcessor regex: 'T' asRegex .
	preProcessor replacement: ''.
	self assert: 'My String' equals: (preProcessor preProcess: ('PreviousContentMTy TStrTingT' asPetitStream position: 15; yourself)) contents ! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testRelativePositionForDoubleInclusion
	|preProcessingParser token |	
	preProcessingParser := (('yoyo' asParser, ('Libellule' asParser info), 'yoyo' asParser) preProcess: 'Y+' asRegex into: '') preProcess: 'T' asRegex  into: ''.
	token := (preProcessingParser parse:  'yTYYoyTYYoLiYYbTeYYlTluTlYYeyToYYyTo') second.
	self assert: token start equals: 11.
	self assert: token stop equals: 28. ! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testRelativePositionForIncludedParser
	|preProcessingParser token |	
	preProcessingParser := (('yoyo' asParser, ('Libellule' asParser info), 'yoyo' asParser) preProcess: 'Y+' asRegex into: '') preProcess: 'T' asRegex  into: ''.
	token := (preProcessingParser parse:  'yToyToLibTelTluTleyToyTo') second.
	self assert: token start equals: 7.
	self assert: token stop equals: 18. ! !

!PPPreProcessingParserRegexTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:03'!
testWithPotentiallyPreprocessedDataAround
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcess: 'T' asRegex into: ''.
	self assert: (preProcessingParser end matches: 'TLibTelTluTleT')! !

PPAbstractParserTest subclass: #PPPreProcessingParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Tests'!

!PPPreProcessingParserTest commentStamp: '<historical>' prior: 0!
A PPPreProcessingParserTest is xxxxxxxxx.

Instance Variables

Double inclusion:
yoyoLiYYbTeYYlTluTlYYeyoyo

==>

yoyoLiYYbeYYllulYYeyoyo

==>

yoyoLibelluleyoyo
!

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:04'!
testBiggerReplacementThanMatching
	|preProcessingParser|	
	preProcessingParser := 'Troll' asParser preProcessor: ('u' asParser ==> [:p | 'll']). 
	self assert: (('Un' asParser, preProcessingParser, 'DeTroy' asParser) end matches: 'UnTrouDeTroy')! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:32'!
testDecomposedEntry
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcessor: ('T' asParser ==> [:p | '']).
	self assert: (preProcessingParser end matches: 'LibTelTluTle')! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:32'!
testDecomposedEntryConsumed
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcessor: ('T' asParser ==> [:p | '']).
	self assert:  ((preProcessingParser, 'yoyo' asParser) matches:  'LibTelTluTleyoyo')! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:32'!
testDoNotConsumeIfContainedParserDoNotMatch
	|preProcessingParser|	
	preProcessingParser :=  'aNonMatchingString' asParser preProcessor: ('T' asParser ==> [:p | '']).
	self assert: ((preProcessingParser optional, 'LibTelTluTle' asParser) matches: 'LibTelTluTle')! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:33'!
testDoubleInclusion
	| preProcessingParser parser |
	preProcessingParser := ('Libellule' asParser preProcessor: ('Y' asParser plus ==> [:p | ''])) preProcessor: ('T' asParser ==> [:p | '']).
	parser :=  'yoyo' asParser, preProcessingParser, 'yoyo' asParser.
	self assert: (parser matches: 'yoyoLiYYbTeYYlTluTlYYeyoyo').! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 14:06'!
testMixingDataToPreProcessAndNot
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcessor: ('T' asParser ==> [:p | '']).
	self assert: (('DoNoTPreProcess' asParser, preProcessingParser, 'NotThisToo' asParser) matches: 'DoNoTPreProcessLibTelTluTleNotThisToo')! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 14:06'!
testPreprocess
	| preProcessor |
	preProcessor := PPrePParserStreamPreProcessor new.
	preProcessor matchingParser: ('T' asParser ==> [:p | '']).
	self assert: 'My String' equals: (preProcessor preProcess: 'MTy TStrTingT' asPetitStream ) contents ! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 14:07'!
testPreprocessWithPreviousContent
	| aStream preProcessor |
	preProcessor := PPrePParserStreamPreProcessor new.
	preProcessor matchingParser: ('T' asParser ==> [:p | '']).
	self assert: 'My String' equals: (preProcessor preProcess: ('PreviousContentMTy TStrTingT' asPetitStream position: 15; yourself)) contents ! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 14:08'!
testRelativePositionForDoubleInclusion
	|preProcessingParser token |	
	preProcessingParser := (('yoyo' asParser, ('Libellule' asParser info), 'yoyo' asParser) preProcessor: ('Y' asParser plus ==> [:p | ''])) preProcessor: ('T' asParser ==> [:p | '']).
	token := (preProcessingParser parse:  'yTYYoyTYYoLiYYbTeYYlTluTlYYeyToYYyTo') second.
	self assert: token start equals: 11.
	self assert: token stop equals: 28. ! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 14:08'!
testRelativePositionForIncludedParser
	|preProcessingParser token |	
	preProcessingParser := (('yoyo' asParser, ('Libellule' asParser info), 'yoyo' asParser) preProcessor: ('Y' asParser plus ==> [:p | ''])) preProcessor: ('T' asParser ==> [:p | '']).
	token := (preProcessingParser parse:  'yToyToLibTelTluTleyToyTo') second.
	self assert: token start equals: 7.
	self assert: token stop equals: 18. ! !

!PPPreProcessingParserTest methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 14:09'!
testWithPotentiallyPreprocessedDataAround
	|preProcessingParser|	
	preProcessingParser := 'Libellule' asParser preProcessor: ('T' asParser ==> [:p | '']).
	self assert: (preProcessingParser end matches: 'TLibTelTluTleT')! !

PPStream subclass: #PPRelativePositionStream
	instanceVariableNames: 'relativeStream transformation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPRelativePositionStream commentStamp: '<historical>' prior: 0!
A PPRelativePositionStream is a specialized stream to get the relative position according to another stream.

Instance Variables
!

!PPRelativePositionStream class methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/9/2012 15:17'!
on: aStream relativeTo: aRelativeStream with: aTransformation
	^ (super on: aStream) relativeStream: aRelativeStream; transformation: aTransformation; yourself! !

!PPRelativePositionStream methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 11:19'!
absolutePosition: aPosition
	"return the absolute position of this stream (related to the original stream)"
	^  relativeStream absolutePosition: (self relativePosition: aPosition ) ! !

!PPRelativePositionStream methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/9/2012 16:58'!
relativePosition: aPosition
	"Return the relative position of this stream to its parent stream"
	^  (transformation relativePosition: aPosition) ! !

!PPRelativePositionStream methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:17'!
relativeStream
	^ relativeStream! !

!PPRelativePositionStream methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:17'!
relativeStream: anObject
	relativeStream := anObject! !

!PPRelativePositionStream methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:17'!
transformation
	^ transformation! !

!PPRelativePositionStream methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:17'!
transformation: anObject
	transformation := anObject! !

!PPStream methodsFor: '*petitPreprocessor-core' stamp: 'guillaumelarcheveque 7/10/2012 11:04'!
absolutePosition
	^ self absolutePosition: position! !

!PPStream methodsFor: '*petitPreprocessor-core' stamp: 'guillaumelarcheveque 7/10/2012 11:04'!
absolutePosition: aPosition
	^ aPosition! !

!PPStream methodsFor: '*petitPreprocessor-core' stamp: ''!
relativePosition
	^ self relativePosition: self position! !

!PPStream methodsFor: '*petitPreprocessor-core' stamp: ''!
relativePosition: aPosition
	^ aPosition ! !

!PPParser methodsFor: '*petitpreprocessor-core' stamp: ''!
info
	"Answer a new parser that transforms the input to an info object, containing evaluation and positions."
	
	^ PPInfoParser on: self! !

!PPParser methodsFor: '*petitpreprocessor-core' stamp: 'guillaumelarcheveque 7/10/2012 10:09'!
preProcess: aRegex into: aReplacement
	^ PPreRegexStreamPreProcessor new setParser: self; regex: aRegex; replacement: aReplacement; yourself! !

!PPParser methodsFor: '*petitpreprocessor-core' stamp: 'gl 9/18/2012 11:18'!
preProcessor: aPParserWithEvaluationBlock
	^ PPrePParserStreamPreProcessor new setParser: self; matchingParser: aPParserWithEvaluationBlock; yourself! !

PPDelegateParser subclass: #PPInfoParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPInfoParser commentStamp: '<historical>' prior: 0!
A PPInfoParser is a containing parser providing the evaluation content and the stream content positions.

To use this, only send "info" to any PPParser:
'My Parser' asParser info

Instance Variables

!

!PPInfoParser methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 6/5/2012 15:46'!
create: element start: startInteger stop: stopInteger
	^ PPInfo on: element start: startInteger stop: stopInteger! !

!PPInfoParser methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 11:20'!
parseOn: aStream
	| start element absoluteStart |
	start := aStream position.
	absoluteStart := aStream absolutePosition.
	element := parser parseOn: aStream.
	element isPetitFailure ifTrue: [
		aStream position: start.
		^ element ].
	^ self create: element start: (absoluteStart + 1) stop: (aStream absolutePosition)! !

PPDelegateParser subclass: #PPreStreamPreProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPreStreamPreProcessor commentStamp: '<historical>' prior: 0!
A PPreTransformer is a processor able to transform a stream into another stream. It will also create the PPre	Transformation to make possible linking the original stream to the processed stream.

Instance Variables
!

PPreStreamPreProcessor subclass: #PPrePParserStreamPreProcessor
	instanceVariableNames: 'matchingParser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPrePParserStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 14:05'!
intervalsToReplace: aStream 
	| position ranges lastPosition keep |
	position := aStream position.
	lastPosition := -1.
	ranges := (matchingParser matchingRangesIn: aStream) select: [:i | keep := (lastPosition < i first). lastPosition := i last. keep ].
	aStream position: position.
	^ ranges! !

!PPrePParserStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:10'!
matchingParser
	^ matchingParser ! !

!PPrePParserStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:10'!
matchingParser: aPParser
	matchingParser := aPParser ! !

!PPrePParserStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:11'!
preProcess: aStream
	"Return a new PPRelativePositionStream which stores all transformations"
	| result transformation |
	result := WriteStream with: ''.
	transformation := PPreIntervalTransformations startingAt: aStream position.
	(self intervalsToReplace: aStream) do: 
		[:aRange | transformation addTransformation: (self write: aStream into: result forRange: aRange).
			 ].
	aStream do: [:c | result nextPut: c].
	^ PPRelativePositionStream on: (result contents) relativeTo: aStream with: transformation! !

!PPrePParserStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'gl 9/18/2012 11:30'!
write: aStream into: anotherStream forRange: aRange
	| aStreamStart anotherStreamStart |
	
	[(aStream position + 1) < (aRange first)] whileTrue: [anotherStream nextPut: aStream next]. 
	
	aStreamStart:= aStream position.
	anotherStreamStart:= anotherStream position.
	anotherStream nextPutAll: (matchingParser parse: (aStream collection copyFrom: aRange first to: aRange last)).
	aStream skip: aRange size.
	
	^ PPreIntervalTransformation from: (Interval from: aStreamStart to: aStream position) to: (Interval from: anotherStreamStart to: anotherStream position)! !

PPreStreamPreProcessor subclass: #PPreRegexStreamPreProcessor
	instanceVariableNames: 'regex replacement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPreRegexStreamPreProcessor commentStamp: '<historical>' prior: 0!
A PPreRegexStreamPreProcessor is a preProcessor based on regular expression.

Instance Variables
	regex:		<Object>
	replacement:		<Object>

regex
	- xxxxx

replacement
	- xxxxx
!

!PPreRegexStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 10:24'!
intervalsToReplace: aStream 
	| position ranges |
	position := aStream position.
	ranges := regex matchingRangesIn: aStream.
	aStream position: position.
	^ ranges! !

!PPreRegexStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 11:20'!
preProcess: aStream
	"Return a new PPRelativePositionStream which stores all transformations"
	| result transformation |
	result := WriteStream with: ''.
	transformation := PPreIntervalTransformations startingAt: aStream position.
	(self intervalsToReplace: aStream) do: 
		[:aRange | transformation addTransformation: (self write: aStream into: result forRange: aRange).
			 ].
	aStream do: [:c | result nextPut: c].
	^ PPRelativePositionStream on: (result contents) relativeTo: aStream with: transformation! !

!PPreRegexStreamPreProcessor methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:08'!
regex
	^ regex! !

!PPreRegexStreamPreProcessor methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:08'!
regex: anObject
	regex := anObject! !

!PPreRegexStreamPreProcessor methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:08'!
replacement
	^ replacement! !

!PPreRegexStreamPreProcessor methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:08'!
replacement: anObject
	replacement := anObject! !

!PPreRegexStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 10:51'!
write: aStream into: anotherStream forRange: aRange
	| aStreamStart anotherStreamStart |
	
	[(aStream position + 1) < (aRange first)] whileTrue: [anotherStream nextPut: aStream next]. 
	
	aStreamStart:= aStream position.
	anotherStreamStart:= anotherStream position.
	
	anotherStream nextPutAll: replacement.
	aStream skip: aRange size.
	
	^ PPreIntervalTransformation from: (Interval from: aStreamStart to: aStream position) to: (Interval from: anotherStreamStart to: anotherStream position)! !

!PPreStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 8/3/2012 15:01'!
parseOn: aStream
	|result preProcessedStream startPosition |
	startPosition := aStream position.
	preProcessedStream := (self preProcess: aStream).
	"call the contained parser"
	result := super parseOn: preProcessedStream.
	result isPetitFailure 
		ifTrue: 
			"restore stream position"
			[aStream position: startPosition.
			result position: (preProcessedStream relativePosition: result position)] 
		ifFalse: 
			"place unprocessed stream at the position corresponding to the processed stream after parsing"
			[aStream position: (preProcessedStream relativePosition)].
	^ result.! !

!PPreStreamPreProcessor methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/6/2012 10:52'!
preProcess: aStream
	^ self subclassResponsibility ! !

!PPFailure methodsFor: '*petitPreprocessor-core' stamp: 'guillaumelarcheveque 8/3/2012 15:03'!
position: aPosition
	position := aPosition ! !

Object subclass: #PPInfo
	instanceVariableNames: 'element start stop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPInfo commentStamp: '<historical>' prior: 0!
A PPInfo represent the informations around a parsing result and also contain this result. It is usefull to get parsing evaluation and stream position.

Instance Variables
	element:		<Object>
	start:		<Object>
	stop:		<Object>

element
	- xxxxx

start
	- xxxxx

stop
	- xxxxx
!

!PPInfo class methodsFor: 'as yet unclassified' stamp: ''!
new
	self error: 'Info can only be created using a dedicated constructor.'! !

!PPInfo class methodsFor: 'as yet unclassified' stamp: ''!
on: anElement start: aStartInteger stop: aStopInteger
	^ self basicNew 
		initializeOn: anElement
		start: aStartInteger stop: aStopInteger! !

!PPInfo methodsFor: 'accessing' stamp: ''!
element
	^ element! !

!PPInfo methodsFor: 'accessing' stamp: ''!
initializeOn: anElement start: aStartInteger stop: aStopInteger
	element := anElement.
	start := aStartInteger.
	stop := aStopInteger! !

!PPInfo methodsFor: 'accessing' stamp: ''!
start
	^ start! !

!PPInfo methodsFor: 'accessing' stamp: ''!
stop
	^ stop! !

Object subclass: #PPreIntervalTransformation
	instanceVariableNames: 'sourceInterval destinationInterval'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPreIntervalTransformation class methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/9/2012 15:22'!
from: anInterval to: anotherInterval
	^ self new sourceInterval: anInterval; destinationInterval: anotherInterval ! !

!PPreIntervalTransformation methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 11:00'!
concerns: aPosition
	^ (self destinationInterval last <= aPosition)! !

!PPreIntervalTransformation methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 10:18'!
decalage
	^ destinationInterval size - sourceInterval size ! !

!PPreIntervalTransformation methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:21'!
destinationInterval
	^ destinationInterval! !

!PPreIntervalTransformation methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:21'!
destinationInterval: anObject
	destinationInterval := anObject! !

!PPreIntervalTransformation methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:42'!
printOn: aStream
	aStream nextPut: $[;
		print: sourceInterval;
		nextPutAll: '=>';
		print: destinationInterval;
		nextPut: $]! !

!PPreIntervalTransformation methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:21'!
sourceInterval
	^ sourceInterval! !

!PPreIntervalTransformation methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:21'!
sourceInterval: anObject
	sourceInterval := anObject! !

Object subclass: #PPreStreamTransformation
	instanceVariableNames: 'relativeStreamStartPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

PPreStreamTransformation subclass: #PPreIntervalTransformations
	instanceVariableNames: 'intervals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitPreprocessor-Core'!

!PPreIntervalTransformations methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/9/2012 15:28'!
addTransformation: aTransformation
	self intervals addLast: aTransformation! !

!PPreIntervalTransformations methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/9/2012 15:06'!
initialize 
	intervals := OrderedCollection new! !

!PPreIntervalTransformations methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:15'!
intervals
	^ intervals! !

!PPreIntervalTransformations methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:15'!
intervals: anObject
	intervals := anObject! !

!PPreIntervalTransformations methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/10/2012 10:48'!
printOn: aStream
	aStream nextPut: ${.
	 self intervals do: [:anInterval | aStream print: anInterval ].
	aStream nextPut: $}! !

!PPreIntervalTransformations methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/10/2012 11:00'!
relativePosition: aPosition
	| decalage |
	decalage:= 0.
	intervals do: [:aTransformation | (aTransformation concerns: aPosition) ifTrue: [decalage := decalage + aTransformation decalage ]].
	^ (aPosition - decalage) + relativeStreamStartPosition! !

!PPreStreamTransformation class methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/9/2012 15:06'!
startingAt: aPosition
	^ self new relativeStreamStartPosition: aPosition ! !

!PPreStreamTransformation methodsFor: 'as yet unclassified' stamp: 'guillaumelarcheveque 7/9/2012 16:55'!
relativePosition: aPosition
	self subclassResponsibility ! !

!PPreStreamTransformation methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:04'!
relativeStreamStartPosition
	^ relativeStreamStartPosition! !

!PPreStreamTransformation methodsFor: 'accessing' stamp: 'guillaumelarcheveque 7/9/2012 15:04'!
relativeStreamStartPosition: anObject
	relativeStreamStartPosition := anObject! !
